LocalMap = class("LocalMap")

function LocalMap:initialize(x,y,dType,difficulty)
self.width = x
self.height = y
self.dungeonType = dType
self.difficulty = difficulty
self.Tile = {}
self.enterX = 1
self.enterY = 1
self.exitX = 2
self.exitY = 2
self:generateDungeonTiles()
self:formatDungeon()
self:countSightable()
self:createExits()
self:createEnemies()
self:setupGraphics()
self.percentSeen = 0
end

function LocalMap:update()
self:updateValues()
self:updateTotalSeen()
end

function LocalMap:draw()
love.graphics.setFont(debugFont)
	for n=-9,9  do
		for m=-9,9 do
			if exists(Game.Player.x+n,Game.Player.y+m,self.Tile) then
				love.graphics.setColor(255,255,255,self.Tile[Game.Player.x+n][Game.Player.y+m].visibility)
					if self.Tile[Game.Player.x+n][Game.Player.y+m].tileType == 1 then
						love.graphics.draw(self.floor,n*32+320,m*32+320)
						-- love.graphics.print(self.Tile[Game.Player.x+n][Game.Player.y+m].value,n*32+320,m*32+320)
					elseif self.Tile[Game.Player.x+n][Game.Player.y+m].tileType == 2 then
						love.graphics.draw(self.wall,n*32+320,m*32+320)				
					end
				love.graphics.setColor(255,255,255,255)	
				-- love.graphics.print(self.Tile[Game.Player.x+n][Game.Player.y+m].visibility,n*32+320,m*32+320)
			end
		end
	end
	
	for n=1,self.width do
		for m=1,self.height do
			if self.Tile[n][m].visibility > 1 then
				if self.Tile[n][m].tileType == 1 then
				love.graphics.print(" ",720+n*4,480+m*4)
				else
				love.graphics.print(".",720+n*4,480+m*4)
				end
			end
		end
	end
	love.graphics.print("x",720+Game.Player.x*4,480+Game.Player.y*4)

end

function LocalMap:updateValues()
	for n=1,self.width do
		for m=1,self.height do
			Game.LocalMap.Tile[n][m].value = 100
		end
	end
	self:knightValues()
	self:princessValues()
end

function LocalMap:knightValues()
	for n=-3,3 do
		for m=-3,3 do
			if exists(Game.Player.Knight.x+n,Game.Player.Knight.y+m,Game.LocalMap.Tile) then
				if Game.LocalMap.Tile[Game.Player.Knight.x+n][Game.Player.Knight.y+m].tileType ~= 2 then
					Game.LocalMap.Tile[Game.Player.Knight.x+n][Game.Player.Knight.y+m].value = 
					round(Game.LocalMap.Tile[Game.Player.Knight.x+n][Game.Player.Knight.y+m].value /(7-(math.abs(n)+math.abs(m))),0)
				end
			end
		end
	end
	if Game.Player.control == "Knight" then
		for n=1, #Game.Player.Knight.lastLocation do
			if exists(Game.Player.Knight.lastLocation[n].x,Game.Player.Knight.lastLocation[n].y,Game.LocalMap.Tile) then
			Game.LocalMap.Tile[Game.Player.Knight.lastLocation[n].x][Game.Player.Knight.lastLocation[n].y].value =Game.LocalMap.Tile[Game.Player.Knight.lastLocation[n].x][Game.Player.Knight.lastLocation[n].y].value - n
			end
		end
	end
	Game.LocalMap.Tile[Game.Player.Knight.x][Game.Player.Knight.y].value =100
end

function LocalMap:princessValues()
	for n=-2,2 do
		for m=-2,2 do
			if exists(Game.Player.Princess.x+n,Game.Player.Princess.y+m,Game.LocalMap.Tile) then
				if Game.LocalMap.Tile[Game.Player.Princess.x+n][Game.Player.Princess.y+m].tileType ~= 2 then
					Game.LocalMap.Tile[Game.Player.Princess.x+n][Game.Player.Princess.y+m].value = 
					round(Game.LocalMap.Tile[Game.Player.Princess.x+n][Game.Player.Princess.y+m].value/(5-(math.abs(n)+math.abs(m))),0)
				end
			end
		end
	end
	if Game.Player.control == "Princess" then
		for n=1, #Game.Player.Princess.lastLocation do
			Game.LocalMap.Tile[Game.Player.Princess.lastLocation[n].x][Game.Player.Princess.lastLocation[n].y].value =Game.LocalMap.Tile[Game.Player.Princess.lastLocation[n].x][Game.Player.Princess.lastLocation[n].y].value - n
		end
	end
	Game.LocalMap.Tile[Game.Player.Princess.x][Game.Player.Princess.y].value =100
end

function LocalMap:generateDungeonTiles()

	for n=1, self.width do
		self.Tile[n] = {}
	end
	for n=1,self.width do
		for m=1, self.height do
			self.Tile[n][m] = {}
			self.Tile[n][m].visibility = 0
			self.Tile[n][m].value = 50
			self.Tile[n][m].hasObject = false
			-- self.Tile[n][m].tileType = "undeclared"
		end
	end
	
end

function LocalMap:formatDungeon()
	if self.dungeonType == "Scattered" then
		self:Scattered()
	elseif self.dungeonType == "Symmetric" then
		self:Symmetric()
	elseif self.dungeonType == "Cavernous" then
		self:Cavernous()
	end

end

function LocalMap:addRooms()
end

function LocalMap:connectRooms()
end

function LocalMap:Scattered()
end

function LocalMap:Cavernous()
-- self.width = #self.Tile
-- self.height = #self.Tile[1]
	for n=1,self.width do
		for m=1,self.height do
			self.Tile[n][m].tileType = math.random(1,2)
		end
	end
	for n=1, self.width do
		self.Tile[n][1].tileType = 2
		self.Tile[n][self.height].tileType = 2
	end
	for m=1, self.height do
		self.Tile[1][m].tileType = 2
		self.Tile[self.width][m].tileType = 2
	end	
	for i=1,2 do
	tempMap = {}
		for n=2,self.width-1 do
			tempMap[n] = {}
			for m=2,self.height-1 do
				-- finds neighbors 
				local filledNeighbors = 0
				for x=-1,1 do
					for y=-1,1 do
						if x==0 and y == 0 then
						break
						else
							if self.Tile[n+x][m+y].tileType == 2 then
								filledNeighbors = filledNeighbors + 1
							end
						end
					end
				end
				-- changes the temp based on neighbors (cellular automata)
				if filledNeighbors < 3 then
					tempMap[n][m] = 1
				elseif filledNeighbors > 5 then
					tempMap[n][m] = 2
				else
					tempMap[n][m] =self.Tile[n][m].tileType
				end
			end
		end
		for n=2,self.width-1 do
			for m=2,self.height-1 do
				self.Tile[n][m].tileType = tempMap[n][m]
			end
		end
	end
end

function LocalMap:createExits()
local exitOptions = {}
	for n=1,self.width do
		if self.Tile[n][2].tileType == 1 then
			exitOptions[#exitOptions+1] = {}
			exitOptions[#exitOptions].x = n
			exitOptions[#exitOptions].y = 1
		end
		if self.Tile[n][self.height-1].tileType == 1 then
			exitOptions[#exitOptions+1] = {}
			exitOptions[#exitOptions].x = n
			exitOptions[#exitOptions].y = table.getn(self.Tile[1])
		end
	end
	for m=1, self.height do
		if self.Tile[2][m].tileType == 1 then
			exitOptions[#exitOptions+1] = {}
			exitOptions[#exitOptions].x = 1
			exitOptions[#exitOptions].y = m
		end
		if 	self.Tile[self.width-1][m].tileType == 1 then
			exitOptions[#exitOptions+1] = {}
			exitOptions[#exitOptions].x = #self.Tile
			exitOptions[#exitOptions].y = m		
		end
	end	
	local enterNum = math.random(1,#exitOptions)
	local exitNum = math.random(1,#exitOptions)
	self.enterX =  exitOptions[enterNum].x
	self.enterY = exitOptions[enterNum].y
	self.exitX = exitOptions[exitNum].x
	self.exitY = exitOptions[exitNum].y
	for n=1,20 do
	local routeDistance = distanceFrom(self.enterX,self.enterY,self.exitX,self.exitY)
	local newExit = math.random(1,#exitOptions)
		if routeDistance< distanceFrom(self.enterX,self.enterY,exitOptions[newExit].x,exitOptions[newExit].y) then
			self.exitX = exitOptions[newExit].x
			self.exitY = exitOptions[newExit].y
			
		end
	end
	self.Tile[self.exitX][self.exitY].tileType = 1 
	self.Tile[self.enterX][self.enterY].tileType = 1 
end

function LocalMap:createEnemies()
self.Enemy = {}
local totalEnemies = (self.width/64)*(self.height/64)
	for n=1,self.difficulty do
		self.Enemy[n] = Enemy:new(difficulty)
	end
end

function LocalMap:setupGraphics()
local hexType = hexTileType[Game.HexMap.Tile[Game.currentHexX][Game.currentHexY].tileType].name
self.floor = love.graphics.newImage("graphics/" .. hexType .. "/floor.png")
self.wall = love.graphics.newImage("graphics/" .. hexType .. "/wall.png")


end


function LocalMap:countSightable()
self.totalSightable = 0
	for n=1,self.width do
		for m=1,self.height do 
			if self.Tile[n][m].tileType == 1 then
				self.totalSightable = self.totalSightable + 1
			end
			if self.Tile[n][m].tileType == 2 then
				if self:checkWallVisibility(n,m) then
					self.totalSightable = self.totalSightable + 1
				end
			end
		end
	end
end

function LocalMap:updateTotalSeen()
local seen = 0
	for n=1,self.width do
		for m=1,self.height do
			if self.Tile[n][m].visibility>0 then
				seen = seen+1
			end
		end
	end
self.percentSeen = round((seen/self.totalSightable)*100,1)
end


function LocalMap:checkWallVisibility(tileX,tileY)
	for x=-1,1 do
		for y=-1,1 do
			if exists(tileX+x,tileY+y,self.Tile) then
				if self.Tile[tileX+x][tileY+y].tileType == 1 then
					return true
				end
			end
		end
	end
	return false
end

function LocalMap:showAll()
	for n=1,self.width do
		for m=1,self.height do
			self.Tile[n][m].visibility = 255
		end
	end
end



